<?php
/**
 * This class handles all hooks for the custom field in a attachment.s
 */

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

class RML_CustomField {
    private static $me = null;
    
    private function __construct() {
        // Silence is golden.
    }
    
    /**
     * When uploading a new item move it to the root.
     * 
     * @hooked add_attachment
     */
    public function add_attachment($attachment_ID) {
        _wp_rml_synchronize_attachment($attachment_ID, _wp_rml_root());
    }
    
    /**
     * When editing a attachment show up a select
     * option to change the parent folder.
     * 
     * @hooked attachment_fields_to_edit
     * @author MatthiasWeb
     * @since 1.0
     */
    public function attachment_fields_to_edit($form_fields, $post) {
        $folderID = wp_attachment_folder($post->ID);

        if (wp_attachment_is_image($post->ID)) {
            $disabled = array(RML_TYPE_COLLECTION);
        }else{
            $disabled = array(RML_TYPE_COLLECTION, RML_TYPE_GALLERY);
        }
        
        // Check move permission
        $selectDisabled = "";
        if ($folderID > 0) {
            $folder = wp_rml_get_by_id($folderID, null, true);
            if ($folder instanceof RML_Folder && $folder->isRestrictFor("mov")) {
                $selectDisabled = 'disabled="disabled"';
            }
        }
        
        // Create form field
        $form_fields['rml_dir'] = array(
        	'label' => __('Folder', RML_TD),
        	'input' => 'html',
        	'html'  => 
        	    '<div class="rml-folder-edit" ' . $selectDisabled . '>' .
        	    RML_Structure::getInstance()->getView()->getHTMLBreadcrumbByID($folderID) . '
        	       <select name="attachments[' . $post->ID . '][rml_folder]" ' . $selectDisabled . '>
	                        ' . RML_Structure::getInstance()->optionsFasade($folderID, $disabled, false) . '
        	       </select>
        	    </div>',
        );
        return $form_fields;
    }
    
    /**
     * When saving a attachment change the parent folder.
     * 
     * @hooked attachment_fields_to_save
     */
    public function attachment_fields_to_save($post, $attachment) {
        if (isset($attachment['rml_folder'])){
            if (RML_Structure::getInstance()->getFolderByID($attachment['rml_folder']) === null) {
                $attachment['rml_folder'] = _wp_rml_root();
            }
            // Get previous folder id
            $updateCount = array(wp_attachment_folder($post["ID"]), $attachment["rml_folder"]);
            
            // Update to new folder id
            _wp_rml_synchronize_attachment($post['ID'], $attachment['rml_folder']);
            
            // Reset the count of both folders manually because we do not use the wp_rml_move api method
            RML_Structure::getInstance()->resetCountCache($updateCount);
        }
        
        return $post;
    }
    
    public static function getInstance() {
        if (self::$me == null) {
                self::$me = new RML_CustomField();
        }
        return self::$me;
    }
}